unit sbarcodeu;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, ExtCtrls;

type

  bar_kind = (sb_jan13,sb_jan8,sb_upc_a,sb_upc_e,sb_code39,sb_2of5std,sb_2of5itf,
             sb_codabar,sb_code128);

  deg_kind = (deg0,deg90,deg180,deg270);

  Sbarcode = class(TImage)
  private
    { Private declarations }
    pmargin_left:integer;
    pmargin_right:integer;
    pmargin_top:integer;
    pmargin_bottom:integer;
    padd_check_degit:boolean;
    pnw_ratio:integer;
    pstart_char:char;
    pstop_char:char;
    pcode_set:char;
    pkind:bar_kind;
    pbase_size:integer;
    pcode:string;
    pdisplay_ocr:boolean;
    pocr_margin_left:integer;
    pocr_margin_right:integer;
    pocr_margin_top:integer;
    pocr_margin_bottom:integer;
    protate:deg_kind;

    posx:integer;

    bbm:tbitmap;  //内部のビットマップ
  protected
    { Protected declarations }
    function make_jan13(s:string):string;
    function make_jan8(s:string):string;
    function make_upc_a(s:string):string;
    function make_upc_e(s:string):string;
    function make_code39(s:string):string;
    function kensaku_code39(s:string):string;
    function calc_cd_code39(s:string):string;
    function make_2_5_std(s:string):string;
    function make_2_5_itf(s:string):string;
    function make_codabar(s:string):string;
    function kensaku_codabar(s:string):string;
    function make_code128(s:string):string;
    function kensaku_code128(i:integer;s:string):string;
    function kensaku_code128_cd(i:integer;s:string):integer;

    procedure draw_code0(bs:string);

    procedure draw_code2(bs:string);
    procedure draw_code3(bs:string);
    procedure draw_code4(bs:string);
    procedure draw_code5(bs:string);

    function ins_string(s:string;i:integer;ins:string):string;

    function read_ocr_font:tfont;
    procedure write_ocr_font(f:tfont);

    procedure rotate_bitmap;

    procedure Resize; override;
  public
    { Public declarations }
    constructor create(aowner:tcomponent); override;
    destructor destroy; override;

    procedure draw_code;

  published
    { Published declarations }
    property margin_left:integer read pmargin_left write pmargin_left;
    property margin_right:integer read pmargin_right write pmargin_right;
    property margin_top:integer read pmargin_top write pmargin_top;
    property margin_bottom:integer read pmargin_bottom write pmargin_bottom;

    property add_check_degit:boolean read padd_check_degit write padd_check_degit;
    property nw_ratio:integer read pnw_ratio write pnw_ratio;
    property start_char:char read pstart_char write pstart_char;
    property stop_char:char read pstop_char write pstop_char;
    property code_set:char read pcode_set write pcode_set;
    property kind:bar_kind read pkind write pkind;
    property base_size:integer read pbase_size write pbase_size;
    property code:string read pcode write pcode;
    property display_ocr:boolean read pdisplay_ocr write pdisplay_ocr;
    property ocr_font:tfont read read_ocr_font write write_ocr_font;

    property ocr_margin_left:integer read pocr_margin_left write pocr_margin_left;
    property ocr_margin_right:integer read pocr_margin_right write pocr_margin_right;
    property ocr_margin_top:integer read pocr_margin_top write pocr_margin_top;
    property ocr_margin_bottom:integer read pocr_margin_bottom write pocr_margin_bottom;
    property rotate:deg_kind read protate write protate;
  end;

const

  //先頭１桁目（付加分）、２文字目～７文字目用変換コード、右６文字分の変換コード
  jan13_table:array[0..9,0..6,0..9] of byte = (
    //0
		 (($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1桁目
		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //2桁目
		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //3桁目
		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //4桁目
		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //5桁目
		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //6桁目
		  ($72,$66,$6c,$42,$5c,$4e,$50,$44,$48,$74)), //右側6文字用

     //1
 		 (($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //2桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //3桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //4桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //5桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //6桁目
 		  ($72,$66,$6c,$42,$5c,$4e,$50,$44,$48,$74)), //右側6文字用

     //2
		 (($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1桁目
		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //2桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //3桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //4桁目
		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //5桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //6桁目
		  ($72,$66,$6c,$42,$5c,$4e,$50,$44,$48,$74)), //右側6文字用

     //3
 		 (($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //2桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //3桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //4桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //5桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //6桁目
 		  ($72,$66,$6c,$42,$5c,$4e,$50,$44,$48,$74)), //右側6文字用

     //4
		 (($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1桁目
   	  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //2桁目
		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //3桁目
		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //4桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //5桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //6桁目
		  ($72,$66,$6c,$42,$5c,$4e,$50,$44,$48,$74)), //右側6文字用

     //5
 		 (($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1桁目
	 	  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //2桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //3桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //4桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //5桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //6桁目
 		  ($72,$66,$6c,$42,$5c,$4e,$50,$44,$48,$74)), //右側6文字用

     //6
		 (($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1桁目
 	 	  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //2桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //3桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //4桁目
		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //5桁目
		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //6桁目
		  ($72,$66,$6c,$42,$5c,$4e,$50,$44,$48,$74)), //右側6文字用

     //7
 		 (($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1桁目
	 	  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //2桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //3桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //4桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //5桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //6桁目
 		  ($72,$66,$6c,$42,$5c,$4e,$50,$44,$48,$74)), //右側6文字用

     //8
 		 (($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1桁目
 	 	  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //2桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //3桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //4桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //5桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //6桁目
		  ($72,$66,$6c,$42,$5c,$4e,$50,$44,$48,$74)), //右側6文字用

     //9
 		 (($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1桁目
	 	  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //2桁目
 	 	  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //3桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //4桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //5桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //6桁目
 		  ($72,$66,$6c,$42,$5c,$4e,$50,$44,$48,$74))  //右側6文字用

    );

  //左４文字分の変換コード、右４文字分の変換コード
  jan8_table:array[0..1,0..9] of byte = (
		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1-4桁目
 		  ($72,$66,$6c,$42,$5c,$4e,$50,$44,$48,$74)   //5-7桁目
    );

  //先頭１桁目（number system character）＋チェックデジット、２文字目～７文字目用変換コード
  upc_e_table:array[0..19,0..5,0..9] of byte = (

     //00
 		 (($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //1桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //2桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //3桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //4桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //5桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b)), //6桁目

     //01
  	 (($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //1桁目
  	  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //2桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //3桁目
  	  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //4桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //5桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b)), //6桁目

     //02
		 (($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //1桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //2桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //3桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //4桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //5桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b)), //6桁目

     //03
  	 (($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //1桁目
  	  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //2桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //3桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //4桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //5桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17)), //6桁目

     //04
   	 (($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //1桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //2桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //3桁目
   	  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //4桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //5桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b)), //6桁目

     //05
  	 (($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //1桁目
  	  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //2桁目
  	  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //3桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //4桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //5桁目
  	  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b)), //6桁目

     //06
  	 (($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //1桁目
  	  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //2桁目
  	  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //3桁目
  	  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //4桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //5桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17)), //6桁目

     //07
  	 (($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //1桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //2桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //3桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //4桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //5桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b)), //6桁目

     //08
  	 (($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //1桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //2桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //3桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //4桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //5桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17)), //6桁目

     //09
  	 (($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //1桁目
		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //2桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //3桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //4桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //5桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17)), //6桁目

     //10
 		 (($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1桁目
		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //2桁目
  	  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //3桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //4桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //5桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17)), //6桁目

     //11
 		 (($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1桁目
		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //2桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //3桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //4桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //5桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17)), //6桁目

     //12
 		 (($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1桁目
		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //2桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //3桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //4桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //5桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17)), //6桁目

     //13
 		 (($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1桁目
		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //2桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //3桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //4桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //5桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b)), //6桁目

     //14
 		 (($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1桁目
  	  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //2桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //3桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //4桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //5桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17)), //6桁目

     //15
 		 (($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //2桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //3桁目
   	  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //4桁目
   	  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //5桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17)), //6桁目

     //16
 		 (($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //2桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //3桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //4桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //5桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b)), //6桁目

     //17
 		 (($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1桁目
  	  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //2桁目
  	  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //3桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //4桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //5桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17)), //6桁目

     //18
 		 (($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1桁目
   	  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //2桁目
  	  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //3桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //4桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //5桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b)), //6桁目

     //19
 		 (($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //1桁目
		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //2桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //3桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b),  //4桁目
 		  ($27,$33,$1b,$21,$1d,$39,$05,$11,$09,$17),  //5桁目
 		  ($0d,$19,$13,$3d,$23,$31,$2f,$3b,$37,$0b))  //6桁目

    );


  //該当文字、ビット文字列
  code39_table:array[0..43,0..1] of string = (
                      ('0','000110100'),
                      ('1','100100001'),
                      ('2','001100001'),
                      ('3','101100000'),
                      ('4','000110001'),
                      ('5','100110000'),
                      ('6','001110000'),
                      ('7','000100101'),
                      ('8','100100100'),
                      ('9','001100100'),
                      ('A','100001001'),
                      ('B','001001001'),
                      ('C','101001000'),
                      ('D','000011001'),
                      ('E','100011000'),
                      ('F','001011000'),
                      ('G','000001101'),
                      ('H','100001100'),
                      ('I','001001100'),
                      ('J','000011100'),
                      ('K','100000011'),
                      ('L','001000011'),
                      ('M','101000010'),
                      ('N','000010011'),
                      ('O','100010010'),
                      ('P','001010010'),
                      ('Q','000000111'),
                      ('R','100000110'),
                      ('S','001000110'),
                      ('T','000010110'),
                      ('U','110000001'),
                      ('V','011000001'),
                      ('W','111000000'),
                      ('X','010010001'),
                      ('Y','110010000'),
                      ('Z','011010000'),
                      ('-','010000101'),
                      ('.','110000100'),
                      (' ','011000100'),
                      ('$','010101000'),
                      ('/','010100010'),
                      ('+','010001010'),
                      ('%','000101010'),
                      ('*','010010100')
                      );

  //ビット文字列
  ind_2_5_table:array[0..9] of string = (
                      '00110',
                      '10001',
                      '01001',
                      '11000',
                      '00101',
                      '10100',
                      '01100',
                      '00011',
                      '10010',
                      '01010'
                      );


  //文字、ビット文字列
  codabar_table:array[0..23,0..1] of string = (
                      ('0','0000011'),
                      ('1','0000110'),
                      ('2','0001001'),
                      ('3','1100000'),
                      ('4','0010010'),
                      ('5','1000010'),
                      ('6','0100001'),
                      ('7','0100100'),
                      ('8','0110000'),
                      ('9','1001000'),
                      ('-','0001100'),
                      ('$','0011000'),
                      ('/','1010001'),
                      (':','1000101'),
                      ('+','0010101'),
                      ('.','1010100'),
                      ('a','0011010'),
                      ('b','0101001'),
                      ('c','0001011'),
                      ('d','0001110'),
                      ('t','0011010'),
                      ('n','0101001'),
                      ('*','0001011'),
                      ('e','0001110')
                      );


  //文字、ビット文字列
  code128_table:array[0..106,0..3] of string = (
                      (' ',' ','00','11011001100'),
                      ('!','!','01','11001101100'),
                      ('"','"','02','11001100110'),
                      ('#','#','03','10010011000'),
                      ('$','$','04','10010001100'),
                      ('%','%','05','10001001100'),
                      ('&','&','06','10011001000'),
                      ('''','''','07','10011000100'),
                      ('(','(','08','10001100100'),
                      (')',')','09','11001001000'),
                      ('*','*','10','11001000100'),
                      ('+','+','11','11000100100'),
                      (',',',','12','10110011100'),
                      ('-','-','13','10011011100'),
                      ('.','.','14','10011001110'),
                      ('/','/','15','10111001100'),
                      ('0','0','16','10011101100'),
                      ('1','1','17','10011100110'),
                      ('2','2','18','11001110010'),
                      ('3','3','19','11001011100'),
                      ('4','4','20','11001001110'),
                      ('5','5','21','11011100100'),
                      ('6','6','22','11001110100'),
                      ('7','7','23','11101101110'),
                      ('8','8','24','11101001100'),
                      ('9','9','25','11100101100'),
                      (':',':','26','11100100110'),
                      (';',';','27','11101100100'),
                      ('<','<','28','11100110100'),
                      ('=','=','29','11100110010'),
                      ('>','>','30','11011011000'),
                      ('?','?','31','11011000110'),
                      ('@','@','32','11000110110'),
                      ('A','A','33','10100011000'),
                      ('B','B','34','10001011000'),
                      ('C','C','35','10001000110'),
                      ('D','D','36','10110001000'),
                      ('E','E','37','10001101000'),
                      ('F','F','38','10001100010'),
                      ('G','G','39','11010001000'),
                      ('H','H','40','11000101000'),
                      ('I','I','41','11000100010'),
                      ('J','J','42','10110111000'),
                      ('K','K','43','10110001110'),
                      ('L','L','44','10001101110'),
                      ('M','M','45','10111011000'),
                      ('N','N','46','10111000110'),
                      ('O','O','47','10001110110'),
                      ('P','P','48','11101110110'),
                      ('Q','Q','49','11010001110'),
                      ('R','R','50','11000101110'),
                      ('S','S','51','11011101000'),
                      ('T','T','52','11011100010'),
                      ('U','U','53','11011101110'),
                      ('V','V','54','11101011000'),
                      ('W','W','55','11101000110'),
                      ('X','X','56','11100010110'),
                      ('Y','Y','57','11101101000'),
                      ('Z','Z','58','11101100010'),
                      ('[','[','59','11100011010'),
                      ('\','\','60','11101111010'),
                      (']',']','61','11001000010'),
                      ('^','^','62','11110001010'),
                      ('_','_','63','10100110000'),
                      (#00,'`','64','10100001100'),
                      (#01,'a','65','10010110000'),
                      (#02,'b','66','10010000110'),
                      (#03,'c','67','10000101100'),
                      (#04,'d','68','10000100110'),
                      (#05,'e','69','10110010000'),
                      (#06,'f','70','10110000100'),
                      (#07,'g','71','10011010000'),
                      (#08,'h','72','10011000010'),
                      (#09,'i','73','10000110100'),
                      (#10,'j','74','10000110010'),
                      (#11,'k','75','11000010010'),
                      (#12,'l','76','11001010000'),
                      (#13,'m','77','11110111010'),
                      (#14,'n','78','11000010100'),
                      (#15,'o','79','10001111010'),
                      (#16,'p','80','10100111100'),
                      (#17,'q','81','10010111100'),
                      (#18,'r','82','10010011110'),
                      (#19,'s','83','10111100100'),
                      (#20,'t','84','10011110100'),
                      (#21,'u','85','10011110010'),
                      (#22,'v','86','11110100100'),
                      (#23,'w','87','11110010100'),
                      (#24,'x','88','11110010010'),
                      (#25,'y','89','11011011110'),
                      (#26,'z','90','11011110110'),
                      (#27,'{','91','11110110110'),
                      (#28,'|','92','10101111000'),
                      (#29,'}','93','10100011110'),
                      (#30,'~','94','10001011110'),
                      (#31,#127,'95','10111101000'),
                      ('FNC3','FNC3','96','10111100010'),
                      ('FNC2','FNC2','97','11110101000'),
                      ('SHIFT','SHIFT','98','11110100010'),
                      ('codeC','codeC','99','10111011110'),
                      ('codeB','FNC4','codeB','10111101110'),
                      ('FNC4','codeA','codeA','11101011110'),
                      ('FNC1','FNC1','FNC1','11110101110'),
                      ('startA','startA','startA','11010000100'),
                      ('startB','startB','startB','11010010000'),
                      ('startC','startC','startC','11010011100'),
                      ('stop','stop','stop','1100011101011')
                      );


procedure Register;

implementation

procedure Register;
begin
  {$I sbarcode_icon.lrs}
  RegisterComponents('mycomponent',[sbarcode]);
end;

constructor Sbarcode.create(aowner:tcomponent);
begin
  inherited create(aowner); {親のコンストラクタを呼ぶ}

  autosize:=false;
  margin_left:=10;
  margin_right:=10;
  margin_top:=2;
  margin_bottom:=2;
  add_check_degit:=true;
  nw_ratio:=3;
  start_char:='a';  //codabar
  stop_char:='a';   //codabar
  code_set:='A';    //code128
  kind:=sb_code128;
  base_size:=2;
  code:='SBARCODE';
  display_ocr:=true;
  ocr_font.Size:=9;

  ocr_margin_left:=10;
  ocr_margin_right:=10;
  ocr_margin_top:=2;
  ocr_margin_bottom:=2;

  rotate:=deg0;

  bbm:=tbitmap.Create;

  picture.Clear;

  draw_code;
end;

destructor Sbarcode.destroy;
begin
  bbm.Free;

  inherited destroy;  {親のデストラクタを呼ぶ}
end;

function Sbarcode.read_ocr_font:Tfont;
begin
  result:=canvas.Font;
end;

procedure Sbarcode.write_ocr_font(f:TFont);
begin
  canvas.font.assign(f);
end;

procedure Sbarcode.draw_code;
var
  r:trect;
  mojikan:integer;
  counter:integer;
begin
  //ビットマップのサイズを親に合わせる
//  showmessage(format('width=%d',[width]));
//  showmessage(format('height=%d',[height]));

  //初期化前なら抜ける
  if(bbm=nil) then
    exit;

  if(rotate in [deg0,deg180]) then
  begin
    bbm.Width:=Width;
    bbm.height:=Height;
	end
  else
  begin
    bbm.Height:=Width;
    bbm.Width:=Height;
	end;

  with bbm do
  begin
	  //白く塗りつぶす
		r.top:=0;
		r.left:=0;
		r.right:=width;
		r.bottom:=height;

		canvas.brush.Color:=clwhite;
		canvas.Brush.Style:=bssolid;
		canvas.FillRect(r);

	  //バーコードを表示する
	  if(kind=sb_jan13) then
	    draw_code0(make_jan13(code))
	  else if(kind=sb_jan8) then
	    draw_code0(make_jan8(code))
	  else if(kind=sb_upc_a) then
	    draw_code0(make_upc_a(code))
	  else if(kind=sb_upc_e) then
	    draw_code0(make_upc_e(code))
	  else if(kind=sb_code39) then
	    draw_code2(make_code39(code))
	  else if(kind=sb_2of5std) then
	    draw_code3(make_2_5_std(code))
	  else if(kind=sb_2of5itf) then
	    draw_code4(make_2_5_itf(code))
	  else if(kind=sb_codabar) then
	    draw_code5(make_codabar(code))
	  else if(kind=sb_code128) then
	    draw_code0(make_code128(code));

	  //文字を表示する
	  if(display_ocr=true) then
	  begin
			canvas.Font.Assign(ocr_font);

	    //文字の入るエリアを白で塗りつぶす
	    r.Left:=margin_left+ocr_margin_left;
	    r.Top:=height-ocr_margin_top-ocr_margin_bottom-canvas.TextHeight('M');
	    r.Right:=margin_left+posx-ocr_margin_right-ocr_margin_left;
	    r.Bottom:=height;

	    canvas.brush.Color:=clwhite;
	//    canvas.brush.Color:=clred;
	  	canvas.Brush.Style:=bssolid;
	  	canvas.FillRect(r);


	    //キャンバスの幅に合わせて文字を分散表示させる
	    mojikan:=((r.Right-r.Left-ocr_margin_left-ocr_margin_right)-canvas.TextWidth(code))
	                div (length(code)-1);

	    for counter:=1 to length(code) do
	    begin
	  		canvas.TextOut(r.left+ocr_margin_left+canvas.TextWidth('A')*(counter-1)+(counter-1)*mojikan,
	                    r.Top+ocr_margin_top,code[counter]);
			end;

	//		canvas.TextOut(r.left+ocr_margin_left,r.Top+ocr_margin_top,code);

		end;
	end;

  //回転して、コピーする
  rotate_bitmap;
end;


procedure Sbarcode.rotate_bitmap;
var
  x,y:integer;
  bmp:tbitmap;
begin
  //削除する
  picture.clear;

  bmp:=tbitmap.Create;
  bmp.Width:=width;
  bmp.Height:=height;
  picture.Bitmap.Assign(bmp);

  case rotate of
    deg0:
      begin
        picture.Bitmap.Assign(bbm);
        {
		    for x:=0 to bbm.Width-1 do
				  for y:=0 to bbm.Height-1 do
		      begin
				    picture.bitmap.Canvas.Pixels[x,y]:=bbm.Canvas.Pixels[x,y];
					end;
        }
			end;
    deg90:
      begin
        try
          picture.Bitmap.BeginUpdate(True);
	        for x:=0 to bbm.Width-1 do
					  for y:=0 to bbm.Height-1 do
	          begin
					    picture.bitmap.Canvas.Pixels[bbm.Height-y-1,x]:=bbm.Canvas.Pixels[x,y];
						end;
				finally
          picture.Bitmap.EndUpdate(false);
				end;
			end;
    deg180:
      begin
        try
          picture.Bitmap.BeginUpdate(True);
			    for x:=0 to bbm.Width-1 do
					  for y:=0 to bbm.Height-1 do
			      begin
					    picture.bitmap.Canvas.Pixels[bbm.Width-x-1,bbm.Height-y-1]:=bbm.Canvas.Pixels[x,y];
						end;
				finally
          picture.Bitmap.EndUpdate(false);
				end;
			end;
    deg270:
      begin
        try
          picture.Bitmap.BeginUpdate(True);
	        for x:=0 to bbm.Width-1 do
					  for y:=0 to bbm.Height-1 do
	          begin
					    picture.bitmap.Canvas.Pixels[y,bbm.Width-x-1]:=bbm.Canvas.Pixels[x,y];
						end;
				finally
          picture.Bitmap.EndUpdate(false);
				end;
			end;
	end;
end;


procedure Sbarcode.draw_code0(bs:string);
var
  counter1:integer;
  counter2:integer;

  tate:integer;

begin
  with bbm do
  begin
	  //バーコードの縦の長さ
	  tate:=Height-margin_top-margin_bottom;

	  posx:=margin_left;

	  for counter1:=1 to length(bs) do
	  begin
	    for counter2:=1 to base_size do
	    begin
	      //1なら黒線、0ならスペース
	      if(bs[counter1]='1') then
	      begin
	        //縦線を引く
	        Canvas.Line(posx,margin_top,posx,tate);
	        inc(posx);
				end
	      else
	        inc(posx);
			end;
		end;
	end;
end;

//code39用
procedure Sbarcode.draw_code2(bs:string);
var
  counter1:integer;
  counter2:integer;
  counter3:integer;

  tate:integer;
  bw:integer;

begin
  with bbm do
  begin

	  //バーコードの縦の長さ
	  tate:=Height-margin_top-margin_bottom;

	  posx:=margin_left; //マージン分を入れる


	  bw:=1;  //1なら黒、0なら白
	  for counter1:=1 to length(bs) do
	  begin
	    //1なら太線、0なら細線
	    if(bs[counter1]='1') then
	    begin
	      for counter3:=1 to nw_ratio do //ナローとワイドの比
	      begin
	        for counter2:=1 to base_size do
	        begin
		        //黒色なら縦線を引く
		        if((bw mod 2)=1) then
			        Canvas.Line(posx,margin_top,posx,tate);
		        inc(posx);
					end;
				end;
			end
	    else
	    begin
	      for counter2:=1 to base_size do
	      begin
		      //黒色なら縦線を引く
		      if((bw mod 2)=1) then
		        Canvas.Line(posx,margin_top,posx,tate);
		      inc(posx);
				end;
			end;
	    inc(bw);

	    //9ビット毎にキャラクターギャップを入れる
	    if((counter1 mod 9)=0) then
	    begin
	      for counter2:=1 to base_size do
	        inc(posx);

	      inc(bw);
			end;
		end;
	end;
end;

//2of5 standard用
procedure Sbarcode.draw_code3(bs:string);
var
  counter1:integer;
  counter2:integer;
  counter3:integer;

  tate:integer;

  p:integer;

begin
  with bbm do
  begin
	  //バーコードの縦の長さ
	  tate:=Height-margin_top-margin_bottom;

	  posx:=margin_left; //マージン分を入れる


	  //始めと終わりの３ビットはスタート・ストップ
	  //通常のビット数は5bitなので、ビットを合わせる

	  p:=3;

	  for counter1:=1 to length(bs) do
	  begin
	    //1なら太線、0なら細線
	    if(bs[counter1]='1') then
	    begin
	      //太線
	      for counter3:=1 to nw_ratio do //ナローとワイドの比
	      begin
	        for counter2:=1 to base_size do
	        begin
		        //縦線を引く
	          Canvas.Line(posx,margin_top,posx,tate);
		        inc(posx);
					end;
				end;
			end
	    else
	    begin
	      //細線
	      for counter2:=1 to base_size do
	      begin
	        //縦線を引く
	        Canvas.Line(posx,margin_top,posx,tate);
	        inc(posx);
				end;
	    end;

	    for counter2:=1 to base_size do
	    begin
	      inc(posx);  //スペース
			end;

	    //5ビット毎にキャラクターギャップを入れる
		  if((p mod 5)=0) then
		  begin
		    for counter2:=1 to base_size do
		      inc(posx);
			end;

	    inc(p);

		end;
	end;
end;


//2 of 5 interleaved(ITF)用
procedure Sbarcode.draw_code4(bs:string);
var
  counter1:integer;
  counter2:integer;
  counter3:integer;

  tate:integer;
  bw:integer;
begin
  with bbm do
  begin
	  //バーコードの縦の長さ
	  tate:=Height-margin_top-margin_bottom;

	  posx:=margin_left; //マージン分を入れる

	  bw:=1;  //1なら黒、0なら白
	  for counter1:=1 to length(bs) do
	  begin
	    //1なら太線、0なら細線
	    if(bs[counter1]='1') then
	    begin
	      for counter3:=1 to nw_ratio do //ナローとワイドの比
	      begin
	        for counter2:=1 to base_size do
	        begin
		        //黒色なら縦線を引く
		        if((bw mod 2)=1) then
			        Canvas.Line(posx,margin_top,posx,tate);
		        inc(posx);
					end;
				end;
			end
	    else
	    begin
	      for counter2:=1 to base_size do
	      begin
		      //黒色なら縦線を引く
		      if((bw mod 2)=1) then
		        Canvas.Line(posx,margin_top,posx,tate);
		      inc(posx);
				end;
			end;
	    inc(bw);

		end;
	end;
end;


//codabar用
procedure Sbarcode.draw_code5(bs:string);
var
  counter1:integer;
  counter2:integer;
  counter3:integer;

  tate:integer;
  bw:integer;
begin
  with bbm do
  begin
	  //バーコードの縦の長さ
	  tate:=Height-margin_top-margin_bottom;

	  posx:=margin_left; //マージン分を入れる

	  bw:=1;  //1なら黒、0なら白
	  for counter1:=1 to length(bs) do
	  begin
	    //1なら太線、0なら細線
	    if(bs[counter1]='1') then
	    begin
	      for counter3:=1 to nw_ratio do //ナローとワイドの比
	      begin
	        for counter2:=1 to base_size do
	        begin
		        //黒色なら縦線を引く
		        if((bw mod 2)=1) then
			        Canvas.Line(posx,margin_top,posx,tate);
		        inc(posx);
					end;
				end;
			end
	    else
	    begin
	      for counter2:=1 to base_size do
	      begin
		      //黒色なら縦線を引く
		      if((bw mod 2)=1) then
		        Canvas.Line(posx,margin_top,posx,tate);
		      inc(posx);
				end;
			end;
	    inc(bw);

	    //7ビット毎にキャラクターギャップを入れる
	    if((counter1 mod 7)=0) then
	    begin
	      for counter2:=1 to base_size do
	      begin
		      //黒色なら縦線を引く
		      if((bw mod 2)=1) then
		        Canvas.Line(posx,margin_top,posx,tate);
		      inc(posx);
				end;

	      inc(bw);
			end;
		end;
	end;
end;


function Sbarcode.make_jan13(s:string):string;
var
  bs:string;
  guusuu:integer;
  kisuu:integer;
  pc:integer;

  counter:integer;
  counter2:integer;
  b:byte;
  i:integer;

  fuka:integer;
begin
  //12桁ならチェックデジットを計算して付け足す
    if(length(s)=12) then
    begin
      guusuu:=0;
      guusuu:=guusuu+strtointdef(s[2],0);
      guusuu:=guusuu+strtointdef(s[4],0);
      guusuu:=guusuu+strtointdef(s[6],0);
      guusuu:=guusuu+strtointdef(s[8],0);
      guusuu:=guusuu+strtointdef(s[10],0);
      guusuu:=guusuu+strtointdef(s[12],0);

      kisuu:=0;
      kisuu:=kisuu+strtointdef(s[1],0);
      kisuu:=kisuu+strtointdef(s[3],0);
      kisuu:=kisuu+strtointdef(s[5],0);
      kisuu:=kisuu+strtointdef(s[7],0);
      kisuu:=kisuu+strtointdef(s[9],0);
      kisuu:=kisuu+strtointdef(s[11],0);

      pc:=10-(((guusuu*3)+kisuu) mod 10);

      if(pc=10) then
        pc:=0;

      s:=s+inttostr(pc);
  	end;

    bs:='';

    //ガードバー(101)
    bs:=bs+'101';

    //付加文字
    fuka:=strtointdef(s[1],0); //先頭の付加文字

    //左6文字
	  for counter:=2 to 7 do
	  begin
	    i:=strtointdef(s[counter],0);

	    b:=jan13_table[fuka,counter-2,i];

	    for counter2:=6 downto 0 do
	    begin
	      if(((b shr counter2) and 1)=1) then
	        bs:=bs+'1'
	      else
	        bs:=bs+'0';
			end;
		end;

    //センターバー
    bs:=bs+'01010';

    //右6文字
	  for counter:=8 to 13 do
	  begin
	    i:=strtointdef(s[counter],0);

	    b:=jan13_table[fuka,6,i];

	    for counter2:=6 downto 0 do
	    begin
	      if(((b shr counter2) and 1)=1) then
	        bs:=bs+'1'
	      else
	        bs:=bs+'0';
			end;
		end;

    //ガードバー(101)
    bs:=bs+'101';

    result:=bs;
end;



function Sbarcode.make_jan8(s:string):string;
var
  bs:string;
  guusuu:integer;
  kisuu:integer;
  pc:integer;

  counter:integer;
  counter2:integer;
  b:byte;
  i:integer;
begin
  //7桁ならチェックデジットを計算して付け足す
    if(length(s)=7) then
    begin
      kisuu:=0;
      kisuu:=kisuu+strtointdef(s[1],0);
      kisuu:=kisuu+strtointdef(s[3],0);
      kisuu:=kisuu+strtointdef(s[5],0);
      kisuu:=kisuu+strtointdef(s[7],0);

      guusuu:=0;
      guusuu:=guusuu+strtointdef(s[2],0);
      guusuu:=guusuu+strtointdef(s[4],0);
      guusuu:=guusuu+strtointdef(s[6],0);

      pc:=10-(((kisuu*3)+guusuu) mod 10);

      if(pc=10) then
        pc:=0;

      s:=s+inttostr(pc);
  	end;

    bs:='';

    //ガードバー(101)
    bs:=bs+'101';

    //左4文字
	  for counter:=1 to 4 do
	  begin
	    i:=strtointdef(s[counter],0);

	    b:=jan8_table[0,i];

	    for counter2:=6 downto 0 do
	    begin
	      if(((b shr counter2) and 1)=1) then
	        bs:=bs+'1'
	      else
	        bs:=bs+'0';
			end;
		end;

    //センターバー
    bs:=bs+'01010';

    //右4文字
	  for counter:=5 to 8 do
	  begin
	    i:=strtointdef(s[counter],0);

	    b:=jan8_table[1,i];

	    for counter2:=6 downto 0 do
	    begin
	      if(((b shr counter2) and 1)=1) then
	        bs:=bs+'1'
	      else
	        bs:=bs+'0';
			end;
		end;

    //ガードバー(101)
    bs:=bs+'101';

    result:=bs;
end;


function Sbarcode.make_upc_a(s:string):string;
var
  bs:string;
  guusuu:integer;
  kisuu:integer;
  pc:integer;

  counter:integer;
  counter2:integer;
  b:byte;
  i:integer;
begin
  //11桁ならチェックデジットを計算して付け足す
    if(length(s)=11) then
    begin
      kisuu:=0;
      kisuu:=kisuu+strtointdef(s[1],0);
      kisuu:=kisuu+strtointdef(s[3],0);
      kisuu:=kisuu+strtointdef(s[5],0);
      kisuu:=kisuu+strtointdef(s[7],0);
      kisuu:=kisuu+strtointdef(s[9],0);
      kisuu:=kisuu+strtointdef(s[11],0);

      guusuu:=0;
      guusuu:=guusuu+strtointdef(s[2],0);
      guusuu:=guusuu+strtointdef(s[4],0);
      guusuu:=guusuu+strtointdef(s[6],0);
      guusuu:=guusuu+strtointdef(s[8],0);
      guusuu:=guusuu+strtointdef(s[10],0);

      pc:=10-(((kisuu*3)+guusuu) mod 10);

      if(pc=10) then
        pc:=0;

      s:=s+inttostr(pc);
  	end;

    bs:='';

    //ガードバー(101)
    bs:=bs+'101';

    //左6文字
	  for counter:=1 to 6 do
	  begin
	    i:=strtointdef(s[counter],0);

	    b:=jan8_table[0,i];

	    for counter2:=6 downto 0 do
	    begin
	      if(((b shr counter2) and 1)=1) then
	        bs:=bs+'1'
	      else
	        bs:=bs+'0';
			end;
		end;

    //センターバー
    bs:=bs+'01010';

    //右6文字
	  for counter:=7 to 12 do
	  begin
	    i:=strtointdef(s[counter],0);

	    b:=jan8_table[1,i];

	    for counter2:=6 downto 0 do
	    begin
	      if(((b shr counter2) and 1)=1) then
	        bs:=bs+'1'
	      else
	        bs:=bs+'0';
			end;
		end;

    //ガードバー(101)
    bs:=bs+'101';

    result:=bs;
end;


function Sbarcode.make_upc_e(s:string):string;
var
  bs:string;
  guusuu:integer;
  kisuu:integer;
  pc:integer;
  bak:string;

  counter:integer;
  counter2:integer;
  b:byte;
  i:integer;

  ns:integer;
  p:integer;
begin
  //7桁ならチェックデジットを計算して付け足す
  if(length(s)=7) then
  begin
    bak:=s;

    case s[7] of
      '0':  s:=ins_string(s,4,'00000');
      '1':  s:=ins_string(s,4,'10000');
      '2':  s:=ins_string(s,4,'20000');
      '3':  s:=ins_string(s,5,'00000');
      '4':  s:=ins_string(s,6,'00000');
      '5':  s:=ins_string(s,7,'00005');
      '6':  s:=ins_string(s,7,'00006');
      '7':  s:=ins_string(s,7,'00007');
      '8':  s:=ins_string(s,7,'00008');
      '9':  s:=ins_string(s,7,'00009');
		end;

    kisuu:=0;
    kisuu:=kisuu+strtointdef(s[1],0);
    kisuu:=kisuu+strtointdef(s[3],0);
    kisuu:=kisuu+strtointdef(s[5],0);
    kisuu:=kisuu+strtointdef(s[7],0);
    kisuu:=kisuu+strtointdef(s[9],0);
    kisuu:=kisuu+strtointdef(s[11],0);

    guusuu:=0;
    guusuu:=guusuu+strtointdef(s[2],0);
    guusuu:=guusuu+strtointdef(s[4],0);
    guusuu:=guusuu+strtointdef(s[6],0);
    guusuu:=guusuu+strtointdef(s[8],0);
    guusuu:=guusuu+strtointdef(s[10],0);

    pc:=10-(((kisuu*3)+guusuu) mod 10);

    if(pc=10) then
      pc:=0;

    s:=bak;

    s:=s+inttostr(pc);
	end;

  bs:='';

  //ガードバー(101)
  bs:=bs+'101';

  //データー6文字
  ns:=strtointdef(s[1],0); //先頭のnumber system character
  pc:=strtointdef(s[8],0); //最後のパリティチェック

  p:=pc+(10*ns);

  for counter:=2 to 7 do
  begin
    i:=strtointdef(s[counter],0);

    b:=upc_e_table[p,counter-2,i];

    for counter2:=6 downto 0 do
    begin
      if(((b shr counter2) and 1)=1) then
        bs:=bs+'1'
      else
        bs:=bs+'0';
		end;
	end;


  //ガードバー(101)
  bs:=bs+'010101';

  result:=bs;
end;


function Sbarcode.make_code39(s:string):string;
var
  bs:string;

  counter:integer;
  c:string;
begin
  bs:='';

  //ガードバー(010010100)  '*'
  bs:=bs+'010010100';

  //データー
  for counter:=1 to length(s) do
  begin
    c:=s[counter];

    bs:=bs+kensaku_code39(c);
  end;

  //チェックデジット
  if(add_check_degit=true) then
    bs:=bs+calc_cd_code39(s);

  //ガードバー(010010100)  '*'
  bs:=bs+'010010100';

  result:=bs;
end;



function Sbarcode.kensaku_code39(s:string):string;
var
  counter:integer;
begin
  result:='';
  for counter:=0 to 43 do
  begin
    if(code39_table[counter,0]=s) then
    begin
      result:=code39_table[counter,1];
      exit;
		end;
	end;
end;

function Sbarcode.calc_cd_code39(s:string):string;
var
  counter:integer;
  counter2:integer;
  cd:integer;
begin
  cd:=0;
  for counter:=1 to length(s) do
  begin
	  for counter2:=0 to 43 do
	  begin
	    if(code39_table[counter2,0]=s[counter]) then
	    begin
        cd:=cd+counter2;
        break;
			end;
		end;
	end;

  cd:=cd mod 43;

  //該当するコードを返す
  result:=code39_table[cd,1];
end;


function Sbarcode.make_2_5_std(s:string):string;
var
  counter:integer;
  i:integer;
  bs:string;
begin
  bs:='';

  //スタートコード(110)
  bs:=bs+'110';

  //データー
  for counter:=1 to length(s) do
  begin
    i:=strtointdef(s[counter],0);

    bs:=bs+ind_2_5_table[i];

  end;

  //ストップコード(101)
  bs:=bs+'101';

  result:=bs;
end;

function Sbarcode.make_2_5_itf(s:string):string;
var
  counter:integer;
  counter2:integer;

  i1:integer;
  i2:integer;
  b1:string;
  b2:string;

  bs:string;
  b:string;

  len:integer;

begin

  len:=length(s) div 2;

  bs:='';

  //データー
  for counter:=1 to len do
  begin
    i1:=strtointdef(s[(counter-1)*2+1],0);
    b1:=ind_2_5_table[i1];

    i2:=strtointdef(s[(counter-1)*2+2],0);
    b2:=ind_2_5_table[i2];

    //インターリーブする
    b:='';
    for counter2:=1 to 5 do
    begin
      b:=b+b1[counter2];
      b:=b+b2[counter2];
		end;

    bs:=bs+b;

	end;

  //スタートコード(0000)
  bs:='0000'+bs;


  //ストップコード(100)
  bs:=bs+'100';

  result:=bs;
end;


function Sbarcode.make_codabar(s:string):string;
var
  counter:integer;
  bs:string;

begin
  bs:='';

  //スタートコード
  bs:=bs+kensaku_codabar(start_char);

  //データー
  for counter:=1 to length(s) do
  begin
    bs:=bs+kensaku_codabar(s[counter]);
	end;

  //ストップコード
  bs:=bs+kensaku_codabar(stop_char);

  result:=bs;
end;

function Sbarcode.kensaku_codabar(s:string):string;
var
  counter:integer;
begin
  result:='';
  for counter:=0 to 23 do
  begin
    if(codabar_table[counter,0]=s) then
    begin
      result:=codabar_table[counter,1];
      exit;
		end;
	end;
end;


function Sbarcode.make_code128(s:string):string;
var
  counter:integer;
  bs:string;

  len:integer;

  cd:integer;
  index:integer;
  w:integer;
begin
  bs:='';

  //スタートコード
  if(code_set='A') then
  begin
    index:=0;
    bs:=bs+kensaku_code128(index,'startA');
    cd:=kensaku_code128_cd(index,'startA');
	end
	else if(code_set='B') then
  begin
    index:=1;
    bs:=bs+kensaku_code128(index,'startB');
    cd:=kensaku_code128_cd(index,'startB');
	end
	else if(code_set='C') then
  begin
    index:=2;
    bs:=bs+kensaku_code128(index,'startC');
    cd:=kensaku_code128_cd(index,'startC');
	end;

  //データー
  w:=1;

  len:=length(s);

  if(index=2) then
  begin
	  for counter:=1 to (len div 2) do
	  begin
	    bs:=bs+kensaku_code128(index,s[(counter-1)*2+1]+s[(counter-1)*2+2]);
      cd:=cd+kensaku_code128_cd(index,s[(counter-1)*2+1]+s[(counter-1)*2+2])*w;
      inc(w);
		end;
	end
  else
  begin
	  for counter:=1 to len do
    begin
	    bs:=bs+kensaku_code128(index,s[counter]);
      cd:=cd+kensaku_code128_cd(index,s[counter])*w;
      inc(w);
		end;
	end;

  //チェックデジット（必須？）
  cd:=cd mod 103;

  bs:=bs+code128_table[cd,3];

	//ストップコード
  bs:=bs+kensaku_code128(index,'stop');

  result:=bs;
end;

function Sbarcode.kensaku_code128(i:integer;s:string):string;
var
  counter:integer;
begin
  result:='';
  for counter:=0 to 106 do
  begin
    if(code128_table[counter,i]=s) then
    begin
      result:=code128_table[counter,3];
      exit;
		end;
	end;
end;

function Sbarcode.kensaku_code128_cd(i:integer;s:string):integer;
var
  counter:integer;
begin
  result:=0;
  for counter:=0 to 106 do
  begin
    if(code128_table[counter,i]=s) then
    begin
      result:=counter;
      exit;
		end;
	end;
end;


function Sbarcode.ins_string(s:string;i:integer;ins:string):string;
var
  r:string;
  l:integer;
begin
  l:=length(s);
  r:=copy(s,1,i-1);
  r:=r+ins;
  r:=r+copy(s,i,l);

  result:=r;
end;


procedure SBarcode.Resize;
begin
  inherited Resize;

  draw_code;
end;

end.
